@@ -80,7 +80,7 @@ public class JNLPFile {
     protected Version specVersion;
 
     /** information */
-    protected List info;
+    protected List<InformationDesc> info;
 
     protected UpdateDesc update;
 
@@ -188,7 +188,7 @@ public class JNLPFile {
         this.fileLocation = location;
 
         this.uniqueKey = Calendar.getInstance().getTimeInMillis() + "-" +
-                         Math.abs(((new java.util.Random()).nextInt())) + "-" +
+                         ((int)(Math.random()*Integer.MAX_VALUE)) + "-" +
                          location;
 
         if (JNLPRuntime.isDebug())
@@ -248,7 +248,7 @@ public class JNLPFile {
 
         try {
             ResourceTracker tracker = new ResourceTracker(false); // no prefetch
-            tracker.addResource(location, version, policy);
+            tracker.addResource(location, version, null, policy);
 
             return tracker.getInputStream(location);
         } catch (Exception ex) {
@@ -256,13 +256,6 @@ public class JNLPFile {
         }
     }
 
-    /**
-     * Returns the JNLP specification versions supported.
-     */
-    public static Version getSupportedVersions() {
-        return Parser.getSupportedVersions();
-    }
-
     /**
      * Returns the JNLP file's title.  This method returns the same
      * value as InformationDesc.getTitle().
@@ -333,11 +326,10 @@ public class JNLPFile {
                 List<Object> result = new ArrayList<Object>();
 
                 for (int i = 0; i < info.size(); i++) {
-                    InformationDesc infoDesc = (InformationDesc) info.get(i);
+                    InformationDesc infoDesc = info.get(i);
 
                     if (localMatches(locale, infoDesc.getLocales()))
-                        if (localMatches(locale, infoDesc.getLocales()))
-                            result.addAll(infoDesc.getItems(key));
+                        result.addAll(infoDesc.getItems(key));
                 }
 
                 return result;
@@ -369,7 +361,7 @@ public class JNLPFile {
     }
 
     /**
-     * Returns the information section of the JNLP file for the
+     * Returns the resources section of the JNLP file for the
      * specified locale, os, and arch.
      */
     public ResourcesDesc getResources(final Locale locale, final String os, final String arch) {
@@ -398,6 +390,33 @@ public class JNLPFile {
         };
     }
 
+    /**
+     * Returns the resources section of the JNLP file as viewed
+     * through the default locale and the os.name and os.arch
+     * properties.
+     * XXX: Before overriding this method or changing its implementation,
+     * read the comment in JNLPFile.getDownloadOptionsForJar(JARDesc).
+     */
+    public ResourcesDesc[] getResourcesDescs() {
+        return getResourcesDescs(defaultLocale, defaultOS, defaultArch);
+    }
+
+    /**
+     * Returns the resources section of the JNLP file for the
+     * specified locale, os, and arch.
+     */
+    public ResourcesDesc[] getResourcesDescs(final Locale locale, final String os, final String arch) {
+        List<ResourcesDesc> matchingResources = new ArrayList<ResourcesDesc>();
+        for (ResourcesDesc rescDesc: resources) {
+            if (localMatches(locale, rescDesc.getLocales())
+                    && stringMatches(os, rescDesc.getOS())
+                    && stringMatches(arch, rescDesc.getArch())) {
+                matchingResources.add(rescDesc);
+            }
+        }
+        return matchingResources.toArray(new ResourcesDesc[0]);
+    }
+
     /**
      * Returns an object of one of the following types: AppletDesc,
      * ApplicationDesc and InstallerDesc
@@ -627,4 +646,32 @@ public class JNLPFile {
         return newVMArgs;
     }
 
+    /**
+     * XXX: this method does a "==" comparison between the input JARDesc and
+     * jars it finds through getResourcesDescs(). If ever the implementation
+     * of that function should change to return copies of JARDescs objects,
+     * then the "jar == aJar" comparison below should change accordingly.
+     * @param jar: the jar whose download options to get.
+     * @return the download options.
+     */
+    public DownloadOptions getDownloadOptionsForJar(JARDesc jar) {
+        boolean usePack = false;
+        boolean useVersion = false;
+        ResourcesDesc[] descs = getResourcesDescs();
+        for (ResourcesDesc desc: descs) {
+            JARDesc[] jars = desc.getJARs();
+            for (JARDesc aJar: jars) {
+                if (jar == aJar) {
+                    if (Boolean.valueOf(desc.getPropertiesMap().get("jnlp.packEnabled"))) {
+                        usePack = true;
+                    }
+                    if (Boolean.valueOf(desc.getPropertiesMap().get("jnlp.versionEnabled"))) {
+                        useVersion = true;
+                    }
+                }
+            }
+        }
+        return new DownloadOptions(usePack, useVersion);
+    }
+
 }
