@@ -21,34 +21,24 @@ import static net.sourceforge.jnlp.runtime.Translator.R;
 import java.applet.Applet;
 import java.awt.Container;
 import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
-import java.io.IOException;
-import java.lang.management.ManagementFactory;
-import java.lang.management.ThreadMXBean;
 import java.lang.reflect.Method;
 import java.net.InetAddress;
 import java.net.URL;
 import java.net.UnknownHostException;
-import java.nio.channels.FileChannel;
-import java.nio.channels.FileLock;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.Map;
 import java.util.jar.JarFile;
 
 import net.sourceforge.jnlp.cache.CacheUtil;
-import net.sourceforge.jnlp.cache.ResourceTracker;
 import net.sourceforge.jnlp.cache.UpdatePolicy;
 import net.sourceforge.jnlp.runtime.AppThreadGroup;
 import net.sourceforge.jnlp.runtime.AppletInstance;
 import net.sourceforge.jnlp.runtime.ApplicationInstance;
-import net.sourceforge.jnlp.runtime.DeploymentConfiguration;
 import net.sourceforge.jnlp.runtime.JNLPClassLoader;
 import net.sourceforge.jnlp.runtime.JNLPRuntime;
 import net.sourceforge.jnlp.services.InstanceExistsException;
 import net.sourceforge.jnlp.services.ServiceUtil;
-import net.sourceforge.jnlp.util.FileUtils;
-import net.sourceforge.jnlp.util.Reflect;
 
 import javax.swing.SwingUtilities;
 import javax.swing.text.html.parser.ParserDelegator;
@@ -86,8 +76,9 @@ public class Launcher {
     /** If the application should call System.exit on fatal errors */
     private boolean exitOnFailure = true;
 
-    /** a lock which is held to indicate that an instance of netx is running */
-    private FileLock fileLock;
+    private ParserSettings parserSettings = new ParserSettings();
+
+    private Map<String, String[]> extra = null;
 
     /**
      * Create a launcher with the runtime's default update policy
@@ -176,6 +167,26 @@ public class Launcher {
         return this.context;
     }
 
+    /**
+     * Set the parser settings to use when the Launcher initiates parsing of
+     * a JNLP file.
+     * @param settings
+     */
+    public void setParserSettings(ParserSettings settings) {
+        parserSettings = settings;
+    }
+
+    /**
+     * Set a map to use when trying to extract extra information, including
+     * arguments, properties and parameters, to be merged into the main JNLP
+     * @param input a map containing extra information to add to the main JNLP.
+     * the values for keys "arguments", "parameters", and "properties" are
+     * used.
+     */
+    public void setInformationToMerge(Map<String, String[]> input) {
+        this.extra = input;
+    }
+
     /**
      * Launches a JNLP file by calling the launch method for the
      * appropriate file type.  The application will be started in
@@ -201,6 +212,10 @@ public class Launcher {
     public ApplicationInstance launch(JNLPFile file, Container cont) throws LaunchException {
         TgThread tg;
 
+        mergeExtraInformation(file, extra);
+
+        JNLPRuntime.markNetxRunning();
+
         //First checks whether offline-allowed tag is specified inside the jnlp
         //file.
         if (!file.getInformation().isOfflineAllowed()) {
@@ -254,6 +269,106 @@ public class Launcher {
         return launch(toFile(location));
     }
 
+    /**
+     * Launches a JNLP file by calling the launch method for the
+     * appropriate file type.
+     *
+     * @param location the URL of the JNLP file to launch
+     * @param fromSource if true, the JNLP file will be re-read from the source
+     * location to get the pristine version
+     * @throws LaunchException if there was an exception
+     * @return the application instance
+     */
+    public ApplicationInstance launch(URL location, boolean fromSource) throws LaunchException {
+        return launch(fromUrl(location, fromSource));
+    }
+
+    /**
+     * Merges extra information into the jnlp file
+     *
+     * @param file the JNLPFile
+     * @param extra extra information to merge into the JNLP file
+     * @throws LaunchException if an exception occurs while extracting
+     * extra information
+     */
+    private void mergeExtraInformation(JNLPFile file, Map<String, String[]> extra) throws LaunchException {
+        if (extra == null) {
+            return;
+        }
+
+        String[] properties = extra.get("properties");
+        if (properties != null) {
+            addProperties(file, properties);
+        }
+
+        String[] arguments = extra.get("arguments");
+        if (arguments != null && file.isApplication()) {
+            addArguments(file, arguments);
+        }
+
+        String[] parameters = extra.get("parameters");
+        if (parameters != null && file.isApplet()) {
+            addParameters(file, parameters);
+        }
+    }
+
+    /**
+     * Add the properties to the JNLP file.
+     * @throws LaunchException if an exception occurs while extracting
+     * extra information
+     */
+    private void addProperties(JNLPFile file, String[] props) throws LaunchException {
+        ResourcesDesc resources = file.getResources();
+
+        for (int i = 0; i < props.length; i++) {
+            // allows empty property, not sure about validity of that.
+            int equals = props[i].indexOf("=");
+            if (equals == -1) {
+                throw launchError(new LaunchException(R("BBadProp", props[i])));
+            }
+
+            String key = props[i].substring(0, equals);
+            String value = props[i].substring(equals + 1, props[i].length());
+
+            resources.addResource(new PropertyDesc(key, value));
+        }
+    }
+
+    /**
+     * Add the params to the JNLP file; only call if file is
+     * actually an applet file.
+     * @throws LaunchException if an exception occurs while extracting
+     * extra information
+     */
+    private void addParameters(JNLPFile file, String[] params) throws LaunchException {
+        AppletDesc applet = file.getApplet();
+
+        for (int i = 0; i < params.length; i++) {
+            // allows empty param, not sure about validity of that.
+            int equals = params[i].indexOf("=");
+            if (equals == -1) {
+                throw launchError(new LaunchException(R("BBadParam", params[i])));
+            }
+
+            String name = params[i].substring(0, equals);
+            String value = params[i].substring(equals + 1, params[i].length());
+
+            applet.addParameter(name, value);
+        }
+    }
+
+    /**
+     * Add the arguments to the JNLP file; only call if file is
+     * actually an application (not installer).
+     */
+    private void addArguments(JNLPFile file, String[] args) {
+        ApplicationDesc app = file.getApplication();
+
+        for (int i = 0; i < args.length; i++) {
+            app.addArgument(args[i]);
+        }
+    }
+
     /**
      * Launches a JNLP file by calling the launch method for the
      * appropriate file type in a different thread.
@@ -330,12 +445,7 @@ public class Launcher {
             List<String> commands = new LinkedList<String>();
 
             // this property is set by the javaws launcher to point to the javaws binary
-            String pathToWebstartBinary = System.getProperty("java.home") +
-                                      File.separatorChar +
-                                      "bin" +
-                                      File.separatorChar +
-                                      "javaws";
-
+            String pathToWebstartBinary = System.getProperty("icedtea-web.bin.location");
             commands.add(pathToWebstartBinary);
             // use -Jargument format to pass arguments to the JVM through the launcher
             for (String arg : vmArgs) {
@@ -360,6 +470,32 @@ public class Launcher {
     /**
      * Returns the JNLPFile for the URL, with error handling.
      */
+    private JNLPFile fromUrl(URL location, boolean fromSource) throws LaunchException {
+        try {
+            JNLPFile file = null;
+
+            file = new JNLPFile(location, parserSettings.isStrict());
+
+            if (fromSource) {
+                // Launches the jnlp file where this file originated.
+                if (file.getSourceLocation() != null) {
+                    file = new JNLPFile(file.getSourceLocation(), parserSettings.isStrict());
+                }
+            }
+            return file;
+        } catch (Exception ex) {
+            if (ex instanceof LaunchException)
+                throw (LaunchException) ex; // already sent to handler when first thrown
+            else
+                // IO and Parse
+                throw launchError(new LaunchException(null, ex, R("LSFatal"), R("LCReadError"), R("LCantRead"), R("LCantReadInfo")));
+        }
+    }
+
+    /**
+     * Returns the JNLPFile for the URL, with error handling.
+     */
+    @Deprecated
     private JNLPFile toFile(URL location) throws LaunchException {
         try {
             JNLPFile file = null;
@@ -395,13 +531,6 @@ public class Launcher {
         if (!file.isApplication())
             throw launchError(new LaunchException(file, null, R("LSFatal"), R("LCClient"), R("LNotApplication"), R("LNotApplicationInfo")));
 
-        markNetxRunning();
-        Runtime.getRuntime().addShutdownHook(new Thread() {
-            public void run() {
-                markNetxStopped();
-            }
-        });
-
         try {
 
             try {
@@ -418,20 +547,7 @@ public class Launcher {
                 return null;
             }
 
-            final int preferredWidth = 500;
-            final int preferredHeight = 400;
-            JNLPSplashScreen splashScreen = null;
-            URL splashImageURL = file.getInformation().getIconLocation(
-                    IconDesc.SPLASH, preferredWidth, preferredHeight);
-            if (splashImageURL != null) {
-                ResourceTracker resourceTracker = new ResourceTracker(true);
-                resourceTracker.addResource(splashImageURL, file.getFileVersion(), updatePolicy);
-                splashScreen = new JNLPSplashScreen(resourceTracker, null, null);
-                splashScreen.setSplashImageURL(splashImageURL);
-                if (splashScreen.isSplashScreenValid()) {
-                    splashScreen.setVisible(true);
-                }
-            }
+            handler.launchInitialized(file);
 
             ApplicationInstance app = createApplication(file);
             app.initialize();
@@ -468,12 +584,7 @@ public class Launcher {
 
             setContextClassLoaderForAllThreads(app.getThreadGroup(), app.getClassLoader());
 
-            if (splashScreen != null) {
-                if (splashScreen.isSplashScreenValid()) {
-                    splashScreen.setVisible(false);
-                }
-                splashScreen.dispose();
-            }
+            handler.launchStarting(app);
 
             main.setAccessible(true);
             main.invoke(null, new Object[] { args });
@@ -585,8 +696,11 @@ public class Launcher {
         try {
             JNLPClassLoader loader = JNLPClassLoader.getInstance(file, updatePolicy);
 
-            if (enableCodeBase || file.getResources().getJARs().length == 0)
+            if (enableCodeBase) {
                 loader.enableCodeBase();
+            } else if (file.getResources().getJARs().length == 0) {
+                throw new ClassNotFoundException("Can't do a codebase look up and there are no jars. Failing sooner rather than later");
+            }
 
             AppThreadGroup group = (AppThreadGroup) Thread.currentThread().getThreadGroup();
 
@@ -625,8 +739,11 @@ public class Launcher {
         try {
             JNLPClassLoader loader = JNLPClassLoader.getInstance(file, updatePolicy);
 
-            if (enableCodeBase || file.getResources().getJARs().length == 0)
+            if (enableCodeBase) {
                 loader.enableCodeBase();
+            } else if (file.getResources().getJARs().length == 0) {
+                throw new ClassNotFoundException("Can't do a codebase look up and there are no jars. Failing sooner rather than later");
+            }
 
             String appletName = file.getApplet().getMainClass();
 
@@ -706,66 +823,6 @@ public class Launcher {
         return null; // chose to continue, or no handler
     }
 
-    /**
-     * Indicate that netx is running by creating the {@link JNLPRuntime#INSTANCE_FILE} and
-     * acquiring a shared lock on it
-     */
-    private void markNetxRunning() {
-        try {
-            String message = "This file is used to check if netx is running";
-
-            File netxRunningFile = new File(JNLPRuntime.getConfiguration()
-                    .getProperty(DeploymentConfiguration.KEY_USER_NETX_RUNNING_FILE));
-            if (!netxRunningFile.exists()) {
-                netxRunningFile.getParentFile().mkdirs();
-                FileUtils.createRestrictedFile(netxRunningFile, true);
-                FileOutputStream fos = new FileOutputStream(netxRunningFile);
-                try {
-                    fos.write(message.getBytes());
-                } finally {
-                    fos.close();
-                }
-            }
-
-            FileInputStream is = new FileInputStream(netxRunningFile);
-            FileChannel channel = is.getChannel();
-            fileLock = channel.tryLock(0, Long.MAX_VALUE, true);
-            if (fileLock != null && fileLock.isShared()) {
-                if (JNLPRuntime.isDebug()) {
-                    System.out.println("Acquired shared lock on " +
-                            netxRunningFile.toString() + " to indicate javaws is running");
-                }
-            } else {
-                fileLock = null;
-            }
-        } catch (IOException e) {
-            e.printStackTrace();
-        }
-
-    }
-
-    /**
-     * Indicate that netx is stopped by releasing the shared lock on
-     * {@link JNLPRuntime#INSTANCE_FILE}.
-     */
-    private void markNetxStopped() {
-        if (fileLock == null) {
-            return;
-        }
-        try {
-            fileLock.release();
-            fileLock.channel().close();
-            fileLock = null;
-            if (JNLPRuntime.isDebug()) {
-                String file = JNLPRuntime.getConfiguration()
-                        .getProperty(DeploymentConfiguration.KEY_USER_NETX_RUNNING_FILE);
-                System.out.println("Release shared lock on " + file);
-            }
-        } catch (IOException e) {
-            e.printStackTrace();
-        }
-    }
-
     /**
      * Do hacks on per-application level to allow different AppContexts to work
      *
@@ -824,7 +881,7 @@ public class Launcher {
                 if (isPlugin) {
                     // Do not display download indicators if we're using gcjwebplugin.
                     JNLPRuntime.setDefaultDownloadIndicator(null);
-                    application = getApplet(file, true, cont);
+                    application = getApplet(file, ((PluginBridge)file).codeBaseLookup(), cont);
                 } else {
                     if (file.isApplication())
                         application = launchApplication(file);
@@ -842,7 +899,7 @@ public class Launcher {
                 exception = ex;
                 // Exit if we can't launch the application.
                 if (exitOnFailure)
-                    System.exit(0);
+                    System.exit(1);
             }
         }
 
