@@ -30,11 +30,13 @@ import java.net.URL;
 import java.net.URLConnection;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.jar.JarOutputStream;
 import java.util.jar.Pack200;
 import java.util.jar.Pack200.Unpacker;
 import java.util.zip.GZIPInputStream;
 
+import net.sourceforge.jnlp.DownloadOptions;
 import net.sourceforge.jnlp.Version;
 import net.sourceforge.jnlp.event.DownloadEvent;
 import net.sourceforge.jnlp.event.DownloadListener;
@@ -91,7 +93,7 @@ public class ResourceTracker {
     //   lock, prefetch, this.resources, each resource, listeners
 
     /** notified on initialization or download of a resource */
-    private static Object lock = new Integer(0); // used to lock static structures
+    private static final Object lock = new Object(); // used to lock static structures
 
     // shortcuts
     private static final int UNINITIALIZED = Resource.UNINITIALIZED;
@@ -117,6 +119,9 @@ public class ResourceTracker {
     /** resources requested to be downloaded */
     private static ArrayList<Resource> queue = new ArrayList<Resource>();
 
+    private static ConcurrentHashMap<Resource, DownloadOptions> downloadOptions =
+        new ConcurrentHashMap<Resource, DownloadOptions>();
+
     /** resource trackers threads are working for (used for load balancing across multi-tracker downloads) */
     private static ArrayList<ResourceTracker> active =
             new ArrayList<ResourceTracker>(); //
@@ -163,7 +168,7 @@ public class ResourceTracker {
      * @param version the resource version
      * @param updatePolicy whether to check for updates if already in cache
      */
-    public void addResource(URL location, Version version, UpdatePolicy updatePolicy) {
+    public void addResource(URL location, Version version, DownloadOptions options, UpdatePolicy updatePolicy) {
         if (location == null)
             throw new IllegalArgumentException("location==null");
 
@@ -177,6 +182,11 @@ public class ResourceTracker {
             resources.add(resource);
         }
 
+        if (options == null) {
+            options = new DownloadOptions(false, false);
+        }
+        downloadOptions.put(resource, options);
+
         // checkCache may take a while (loads properties file).  this
         // should really be synchronized on resources, but the worst
         // case should be that the resource will be updated once even
@@ -619,9 +629,13 @@ public class ResourceTracker {
     private void downloadResource(Resource resource) {
         resource.fireDownloadEvent(); // fire DOWNLOADING
 
+        CacheEntry origEntry = new CacheEntry(resource.location, resource.downloadVersion); // This is where the jar file will be.
+        origEntry.lock();
+
         try {
             // create out second in case in does not exist
-            URLConnection con = getVersionedResourceURL(resource).openConnection();
+            URL realLocation = resource.getDownloadLocation();
+            URLConnection con = realLocation.openConnection();
             con.addRequestProperty("Accept-Encoding", "pack200-gzip, gzip");
 
             con.connect();
@@ -636,71 +650,90 @@ public class ResourceTracker {
             String contentEncoding = con.getContentEncoding();
 
             if (JNLPRuntime.isDebug()) {
-                System.err.println("Content encoding for " + resource.location + ": "
-                        + contentEncoding);
+                System.err.println("Downloading" + resource.location + " using " +
+                        realLocation + " (encoding : " + contentEncoding + ")");
+
             }
 
-            if (contentEncoding != null) {
-                if (contentEncoding.equals("gzip")) {
-                    downloadLocation = new URL(downloadLocation.toString() + ".gz");
-                } else if (contentEncoding.equals("pack200-gzip")) {
-                    downloadLocation = new URL(downloadLocation.toString() + ".pack.gz");
+            boolean packgz = "pack200-gzip".equals(contentEncoding) ||
+                                realLocation.getPath().endsWith(".pack.gz");
+            boolean gzip = "gzip".equals(contentEncoding);
+
+            // It's important to check packgz first. If a stream is both
+            // pack200 and gz encoded, then con.getContentEncoding() could
+            // return ".gz", so if we check gzip first, we would end up
+            // treating a pack200 file as a jar file.
+            if (packgz) {
+                downloadLocation = new URL(downloadLocation.toString() + ".pack.gz");
+            } else if (gzip) {
+                downloadLocation = new URL(downloadLocation.toString() + ".gz");
+            }
+
+            File downloadLocationFile = CacheUtil.getCacheFile(downloadLocation, resource.downloadVersion);
+            CacheEntry downloadEntry = new CacheEntry(downloadLocation, resource.downloadVersion);
+            File finalFile = CacheUtil.getCacheFile(resource.location, resource.downloadVersion); // This is where extracted version will be, or downloaded file if not compressed.
+
+            if (!finalFile.exists()) {
+                // Make sure we don't re-download the file. however it will wait as if it was downloading.
+                // (This is fine because file is not ready yet anyways)
+                byte buf[] = new byte[1024];
+                int rlen;
+
+                InputStream in = new BufferedInputStream(con.getInputStream());
+                OutputStream out = CacheUtil.getOutputStream(downloadLocation, resource.downloadVersion);
+
+                while (-1 != (rlen = in.read(buf))) {
+                    resource.transferred += rlen;
+                    out.write(buf, 0, rlen);
                 }
-            }
 
-            InputStream in = new BufferedInputStream(con.getInputStream());
-            OutputStream out = CacheUtil.getOutputStream(downloadLocation, resource.downloadVersion);
-            byte buf[] = new byte[1024];
-            int rlen;
+                in.close();
+                out.close();
 
-            while (-1 != (rlen = in.read(buf))) {
-                resource.transferred += rlen;
-                out.write(buf, 0, rlen);
-            }
+                // explicitly close the URLConnection.
+                if (con instanceof HttpURLConnection)
+                    ((HttpURLConnection) con).disconnect();
 
-            in.close();
-            out.close();
-
-            // explicitly close the URLConnection.
-            if (con instanceof HttpURLConnection)
-                ((HttpURLConnection) con).disconnect();
-
-            /*
-             * If the file was compressed, uncompress it.
-             */
-            if (contentEncoding != null) {
-                if (contentEncoding.equals("gzip")) {
+                /*
+                 * If the file was compressed, uncompress it.
+                 */
+                if (packgz) {
                     GZIPInputStream gzInputStream = new GZIPInputStream(new FileInputStream(CacheUtil
                             .getCacheFile(downloadLocation, resource.downloadVersion)));
                     InputStream inputStream = new BufferedInputStream(gzInputStream);
 
-                    BufferedOutputStream outputStream = new BufferedOutputStream(
-                            new FileOutputStream(CacheUtil.getCacheFile(resource.location,
-                                    resource.downloadVersion)));
+                    JarOutputStream outputStream = new JarOutputStream(new FileOutputStream(CacheUtil
+                            .getCacheFile(resource.location, resource.downloadVersion)));
+
+                    Unpacker unpacker = Pack200.newUnpacker();
+                    unpacker.unpack(inputStream, outputStream);
+
+                    outputStream.close();
+                    inputStream.close();
+                    gzInputStream.close();
+                } else if (gzip) {
+                    GZIPInputStream gzInputStream = new GZIPInputStream(new FileInputStream(CacheUtil
+                            .getCacheFile(downloadLocation, resource.downloadVersion)));
+                    InputStream inputStream = new BufferedInputStream(gzInputStream);
+
+                    BufferedOutputStream outputStream = new BufferedOutputStream(new FileOutputStream(CacheUtil
+                            .getCacheFile(resource.location, resource.downloadVersion)));
 
                     while (-1 != (rlen = inputStream.read(buf))) {
                         outputStream.write(buf, 0, rlen);
                     }
 
-                    outputStream.close();
-                    inputStream.close();
-                    gzInputStream.close();
-
-                } else if (contentEncoding.equals("pack200-gzip")) {
-                    GZIPInputStream gzInputStream = new GZIPInputStream(new FileInputStream(
-                            CacheUtil.getCacheFile(downloadLocation, resource.downloadVersion)));
-                    InputStream inputStream = new BufferedInputStream(gzInputStream);
-
-                    JarOutputStream outputStream = new JarOutputStream(new FileOutputStream(
-                            CacheUtil.getCacheFile(resource.location, resource.downloadVersion)));
-
-                    Unpacker unpacker = Pack200.newUnpacker();
-                    unpacker.unpack(inputStream, outputStream);
-
                     outputStream.close();
                     inputStream.close();
                     gzInputStream.close();
                 }
+            } else {
+                resource.transferred = downloadLocationFile.length();
+            }
+
+            if (!downloadLocationFile.getPath().equals(finalFile.getPath())) {
+                downloadEntry.markForDelete();
+                downloadEntry.store();
             }
 
             resource.changeStatus(DOWNLOADING, DOWNLOADED);
@@ -717,6 +750,8 @@ public class ResourceTracker {
                 lock.notifyAll(); // wake up wait's to check for completion
             }
             resource.fireDownloadEvent(); // fire ERROR
+        } finally {
+            origEntry.unlock();
         }
     }
 
@@ -727,15 +762,32 @@ public class ResourceTracker {
     private void initializeResource(Resource resource) {
         resource.fireDownloadEvent(); // fire CONNECTING
 
+        CacheEntry entry = new CacheEntry(resource.location, resource.requestVersion);
+        entry.lock();
+
         try {
             File localFile = CacheUtil.getCacheFile(resource.location, resource.downloadVersion);
 
             // connect
-            URLConnection connection = getVersionedResourceURL(resource).openConnection(); // this won't change so should be okay unsynchronized
+            URL finalLocation = findBestUrl(resource);
+            resource.setDownloadLocation(finalLocation);
+            URLConnection connection = finalLocation.openConnection(); // this won't change so should be okay unsynchronized
             connection.addRequestProperty("Accept-Encoding", "pack200-gzip, gzip");
 
             int size = connection.getContentLength();
             boolean current = CacheUtil.isCurrent(resource.location, resource.requestVersion, connection) && resource.getUpdatePolicy() != UpdatePolicy.FORCE;
+            if (!current) {
+                if (entry.isCached()) {
+                    entry.markForDelete();
+                    entry.store();
+                    // Old entry will still exist. (but removed at cleanup)
+                    localFile = CacheUtil.makeNewCacheFile(resource.location, resource.downloadVersion);
+                    CacheEntry newEntry = new CacheEntry(resource.location, resource.requestVersion);
+                    newEntry.lock();
+                    entry.unlock();
+                    entry = newEntry;
+                }
+            }
 
             synchronized (resource) {
                 resource.localFile = localFile;
@@ -749,7 +801,6 @@ public class ResourceTracker {
             }
 
             // update cache entry
-            CacheEntry entry = new CacheEntry(resource.location, resource.requestVersion);
             if (!current)
                 entry.initialize(connection);
 
@@ -773,31 +824,52 @@ public class ResourceTracker {
                 lock.notifyAll(); // wake up wait's to check for completion
             }
             resource.fireDownloadEvent(); // fire ERROR
+        } finally {
+            entry.unlock();
         }
     }
 
     /**
-     * Returns the versioned url for a resource
-     * @param resource the resource to get the url for
+     * Returns the best URL to use for downloading the resource
+     *
+     * @param resource the resource
+     * @return a URL or null
      */
-    private URL getVersionedResourceURL(Resource resource) {
-        String actualLocation = resource.location.getProtocol() + "://"
-                + resource.location.getHost();
-        if (resource.location.getPort() != -1) {
-            actualLocation += ":" + resource.location.getPort();
+    private URL findBestUrl(Resource resource) {
+        DownloadOptions options = downloadOptions.get(resource);
+        if (options == null) {
+            options = new DownloadOptions(false, false);
         }
-        actualLocation += resource.location.getPath();
-        if (resource.requestVersion != null
-                && resource.requestVersion.isVersionId()) {
-            actualLocation += "?version-id=" + resource.requestVersion;
+
+        List<URL> urls = new ResourceUrlCreator(resource, options).getUrls();
+        if (JNLPRuntime.isDebug()) {
+            System.err.println("All possible urls for " +
+                    resource.toString() + " : " + urls);
         }
-        URL versionedURL;
-        try {
-            versionedURL = new URL(actualLocation);
-        } catch (MalformedURLException e) {
-            return resource.location;
+        URL bestUrl = null;
+        for (int i = 0; i < urls.size(); i++) {
+            URL url = urls.get(i);
+            try {
+                URLConnection connection = url.openConnection();
+                connection.addRequestProperty("Accept-Encoding", "pack200-gzip, gzip");
+                if (connection instanceof HttpURLConnection) {
+                    HttpURLConnection con = (HttpURLConnection)connection;
+                    int responseCode = con.getResponseCode();
+                    if (responseCode == -1 || responseCode < 200 || responseCode >= 300) {
+                        continue;
+                    }
+                }
+                if (JNLPRuntime.isDebug()) {
+                    System.err.println("best url for " + resource.toString() + " is " + url.toString());
+                }
+                bestUrl = url;
+                break;
+            } catch (IOException e) {
+                // continue
+            }
         }
-        return versionedURL;
+
+        return bestUrl;
     }
 
     /**
