@@ -1,5 +1,5 @@
 /* ControlPanel.java -- Display the control panel for modifying deployment settings.
-Copyright (C) 2010 Red Hat
+Copyright (C) 2011 Red Hat
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,10 +18,14 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 package net.sourceforge.jnlp.controlpanel;
 
+import static net.sourceforge.jnlp.runtime.Translator.R;
+
 import java.awt.BorderLayout;
 import java.awt.CardLayout;
 import java.awt.Dimension;
 import java.awt.FlowLayout;
+import java.awt.Font;
+import java.awt.GridLayout;
 import java.awt.Image;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
@@ -41,14 +45,15 @@ import javax.swing.JPanel;
 import javax.swing.JScrollPane;
 import javax.swing.SwingConstants;
 import javax.swing.SwingUtilities;
+import javax.swing.UIManager;
 import javax.swing.WindowConstants;
 import javax.swing.border.EmptyBorder;
 import javax.swing.event.ListSelectionEvent;
 import javax.swing.event.ListSelectionListener;
 
-import net.sourceforge.jnlp.runtime.DeploymentConfiguration;
-import net.sourceforge.jnlp.runtime.JNLPRuntime;
+import net.sourceforge.jnlp.config.DeploymentConfiguration;
 import net.sourceforge.jnlp.runtime.Translator;
+import net.sourceforge.jnlp.security.KeyStores;
 import net.sourceforge.jnlp.security.viewer.CertificatePane;
 
 /**
@@ -66,7 +71,7 @@ public class ControlPanel extends JFrame {
      * @author @author Andrew Su (asu@redhat.com, andrew.su@utoronto.ca)
      * 
      */
-    private class SettingsPanel {
+    private static class SettingsPanel {
         final String value;
         final JPanel panel;
 
@@ -79,6 +84,7 @@ public class ControlPanel extends JFrame {
             return panel;
         }
 
+        @Override
         public String toString() {
             return value;
         }
@@ -86,11 +92,6 @@ public class ControlPanel extends JFrame {
 
     private DeploymentConfiguration config = null;
 
-    /*
-     * actual configuration options
-     */
-    private String configBrowserCommand = null;
-
     /**
      * Creates a new instance of the ControlPanel.
      * 
@@ -104,17 +105,57 @@ public class ControlPanel extends JFrame {
 
         this.config = config;
 
+        JPanel topPanel = createTopPanel();
         JPanel mainPanel = createMainSettingsPanel();
         JPanel buttonPanel = createButtonPanel();
 
+        add(topPanel, BorderLayout.PAGE_START);
         add(mainPanel, BorderLayout.CENTER);
-        add(buttonPanel, BorderLayout.SOUTH);
+        add(buttonPanel, BorderLayout.PAGE_END);
         setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
         pack();
         setMinimumSize(getPreferredSize());
         setResizable(false);
     }
 
+    private JPanel createTopPanel() {
+        Font currentFont = null;
+        JLabel about = new JLabel(R("CPMainDescriptionShort"));
+        currentFont = about.getFont();
+        about.setFont(currentFont.deriveFont(currentFont.getSize2D() + 2));
+        currentFont = about.getFont();
+        about.setFont(currentFont.deriveFont(Font.BOLD));
+
+        JLabel description = new JLabel(R("CPMainDescriptionLong"));
+        description.setBorder(new EmptyBorder(2, 0, 2, 0));
+
+        JPanel descriptionPanel = new JPanel(new GridLayout(0, 1));
+        descriptionPanel.setBackground(UIManager.getColor("TextPane.background"));
+        descriptionPanel.add(about);
+        descriptionPanel.add(description);
+
+        JLabel image = new JLabel();
+
+        ClassLoader cl = getClass().getClassLoader();
+        if (cl == null) {
+            cl = ClassLoader.getSystemClassLoader();
+        }
+
+        try {
+            URL imgUrl = cl.getResource("net/sourceforge/jnlp/resources/netx-icon.png");
+            image.setIcon(new ImageIcon(ImageIO.read(imgUrl)));
+        } catch (IOException e) {
+            e.printStackTrace();
+        }
+
+        JPanel topPanel = new JPanel(new BorderLayout());
+        topPanel.setBackground(UIManager.getColor("TextPane.background"));
+        topPanel.add(descriptionPanel, BorderLayout.LINE_START);
+        topPanel.add(image, BorderLayout.LINE_END);
+        topPanel.setBorder(new EmptyBorder(10, 10, 10, 10));
+        return topPanel;
+    }
+
     /**
      * Creates the "ok" "apply" and "cancel" buttons.
      * 
@@ -177,16 +218,16 @@ public class ControlPanel extends JFrame {
      */
     private JPanel createMainSettingsPanel() {
 
-        loadConfiguration();
-
         SettingsPanel[] panels = new SettingsPanel[] { new SettingsPanel(Translator.R("CPTabAbout"), createAboutPanel()),
                 new SettingsPanel(Translator.R("CPTabCache"), createCacheSettingsPanel()),
                 new SettingsPanel(Translator.R("CPTabCertificate"), createCertificatesSettingsPanel()),
-                //                new SettingsPanel(Translator.R("CPTabClassLoader"), createClassLoaderSettingsPanel()),
+                // TODO: This is commented out since this is not implemented yet
+                // new SettingsPanel(Translator.R("CPTabClassLoader"), createClassLoaderSettingsPanel()),
                 new SettingsPanel(Translator.R("CPTabDebugging"), createDebugSettingsPanel()),
                 new SettingsPanel(Translator.R("CPTabDesktopIntegration"), createDesktopSettingsPanel()),
                 new SettingsPanel(Translator.R("CPTabNetwork"), createNetworkSettingsPanel()),
-                new SettingsPanel(Translator.R("CPTabRuntimes"), createRuntimesSettingsPanel()),
+                // TODO: This is commented out since this is not implemented yet
+                // new SettingsPanel(Translator.R("CPTabRuntimes"), createRuntimesSettingsPanel()),
                 new SettingsPanel(Translator.R("CPTabSecurity"), createSecuritySettingsPanel()), };
 
         // Add panels.
@@ -273,7 +314,12 @@ public class ControlPanel extends JFrame {
         JPanel notImplementedPanel = new NamedBorderPanel("Unimplemented");
         notImplementedPanel.setLayout(new BorderLayout());
 
-        URL imgUrl = getClass().getClassLoader().getResource("net/sourceforge/jnlp/resources/warning.png");
+        ClassLoader cl = getClass().getClassLoader();
+        if (cl == null) {
+            cl = ClassLoader.getSystemClassLoader();
+        }
+
+        URL imgUrl = cl.getResource("net/sourceforge/jnlp/resources/warning.png");
         Image img;
         try {
             img = ImageIO.read(imgUrl);
@@ -286,16 +332,6 @@ public class ControlPanel extends JFrame {
         return notImplementedPanel;
     }
 
-    /**
-     * Get the location of the browser.
-     */
-    private void loadConfiguration() {
-        configBrowserCommand = config.getProperty("deployment.browser.path");
-        if (configBrowserCommand == null) {
-            configBrowserCommand = "";
-        }
-    }
-
     /**
      * Save the configuration changes.
      */
@@ -308,14 +344,27 @@ public class ControlPanel extends JFrame {
     }
 
     public static void main(String[] args) throws Exception {
-        JNLPRuntime.initialize(true);
-        final DeploymentConfiguration config = JNLPRuntime.getConfiguration();
+        final DeploymentConfiguration config = new DeploymentConfiguration();
         try {
             config.load();
         } catch (ConfigurationException e) {
-            // TODO Auto-generated catch block
+            // FIXME inform user about this and exit properly
+            // the only known condition under which this can happen is when a
+            // required system configuration file is not found
+
+            // if configuration is not loaded, we will get NullPointerExceptions
+            // everywhere
             e.printStackTrace();
         }
+
+        try {
+            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
+        } catch (Exception e) {
+            // ignore; not a big deal
+        }
+
+        KeyStores.setConfiguration(config);
+
         SwingUtilities.invokeLater(new Runnable() {
             @Override
             public void run() {
