@@ -39,12 +39,12 @@ import java.util.HashMap;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
-import java.util.Random;
 import java.util.TreeSet;
 import java.util.Vector;
 import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
 
+import net.sourceforge.jnlp.DownloadOptions;
 import net.sourceforge.jnlp.ExtensionDesc;
 import net.sourceforge.jnlp.JARDesc;
 import net.sourceforge.jnlp.JNLPFile;
@@ -57,8 +57,8 @@ import net.sourceforge.jnlp.Version;
 import net.sourceforge.jnlp.cache.CacheUtil;
 import net.sourceforge.jnlp.cache.ResourceTracker;
 import net.sourceforge.jnlp.cache.UpdatePolicy;
-import net.sourceforge.jnlp.security.SecurityWarning;
-import net.sourceforge.jnlp.security.SecurityWarning.AccessType;
+import net.sourceforge.jnlp.security.SecurityDialogs;
+import net.sourceforge.jnlp.security.SecurityDialogs.AccessType;
 import net.sourceforge.jnlp.tools.JarSigner;
 import net.sourceforge.jnlp.util.FileUtils;
 import sun.misc.JarIndex;
@@ -144,6 +144,9 @@ public class JNLPClassLoader extends URLClassLoader {
     /** Map of specific codesources to securitydesc */
     private HashMap<URL, SecurityDesc> jarLocationSecurityMap =
             new HashMap<URL, SecurityDesc>();
+    
+    /** Loader for codebase (which is a path, rather than a file) */
+    private CodeBaseClassLoader codeBaseLoader;
 
     /**
      * Create a new JNLPClassLoader from the specified file.
@@ -163,11 +166,11 @@ public class JNLPClassLoader extends URLClassLoader {
         // initialize extensions
         initializeExtensions();
 
+        initializeResources();
+
         // initialize permissions
         initializePermissions();
 
-        initializeResources();
-
         setSecurity();
 
         installShutdownHooks();
@@ -275,10 +278,12 @@ public class JNLPClassLoader extends URLClassLoader {
 
         try {
 
-            // If base loader is null, or the baseloader's file and this
-            // file is different, initialize a new loader
+
+            // A null baseloader implies that no loader has been created 
+            // for this codebase/jnlp yet. Create one.
             if (baseLoader == null ||
-                        !baseLoader.getJNLPFile().getFileLocation().equals(file.getFileLocation())) {
+                    (file.isApplication() && 
+                     !baseLoader.getJNLPFile().getFileLocation().equals(file.getFileLocation()))) {
 
                 loader = new JNLPClassLoader(file, policy);
 
@@ -288,7 +293,7 @@ public class JNLPClassLoader extends URLClassLoader {
 
                 if (extLoader != null && extLoader != loader) {
                     if (loader.signing && !extLoader.signing)
-                        if (!SecurityWarning.showNotAllSignedWarningDialog(file))
+                        if (!SecurityDialogs.showNotAllSignedWarningDialog(file))
                             throw new LaunchException(file, null, R("LSFatal"), R("LCClient"), R("LSignedAppJarUsingUnsignedJar"), R("LSignedAppJarUsingUnsignedJarInfo"));
 
                     loader.merge(extLoader);
@@ -302,6 +307,13 @@ public class JNLPClassLoader extends URLClassLoader {
 
             } else {
                 // if key is same and locations match, this is the loader we want
+                if (!file.isApplication()) {
+                    // If this is an applet, we do need to consider its loader
+                    loader = new JNLPClassLoader(file, policy);
+
+                    if (baseLoader != null)
+                        baseLoader.merge(loader);
+                }
                 loader = baseLoader;
             }
 
@@ -406,6 +418,7 @@ public class JNLPClassLoader extends URLClassLoader {
 
             tracker.addResource(jars[i].getLocation(),
                                 jars[i].getVersion(),
+                                getDownloadOptionsForJar(jars[i]),
                                 jars[i].isCacheable() ? JNLPRuntime.getDefaultUpdatePolicy() : UpdatePolicy.FORCE
                                );
         }
@@ -434,7 +447,7 @@ public class JNLPClassLoader extends URLClassLoader {
                 signing = true;
 
                 if (!js.allJarsSigned() &&
-                                    !SecurityWarning.showNotAllSignedWarningDialog(file))
+                                    !SecurityDialogs.showNotAllSignedWarningDialog(file))
                     throw new LaunchException(file, null, R("LSFatal"), R("LCClient"), R("LSignedAppJarUsingUnsignedJar"), R("LSignedAppJarUsingUnsignedJarInfo"));
 
                 //user does not trust this publisher
@@ -501,7 +514,7 @@ public class JNLPClassLoader extends URLClassLoader {
 
     private void checkTrustWithUser(JarSigner js) throws LaunchException {
         if (!js.getRootInCacerts()) { //root cert is not in cacerts
-            boolean b = SecurityWarning.showCertWarningDialog(
+            boolean b = SecurityDialogs.showCertWarningDialog(
                     AccessType.UNVERIFIED, file, js);
             if (!b)
                 throw new LaunchException(null, null, R("LSFatal"),
@@ -509,10 +522,10 @@ public class JNLPClassLoader extends URLClassLoader {
         } else if (js.getRootInCacerts()) { //root cert is in cacerts
             boolean b = false;
             if (js.noSigningIssues())
-                b = SecurityWarning.showCertWarningDialog(
+                b = SecurityDialogs.showCertWarningDialog(
                         AccessType.VERIFIED, file, js);
             else if (!js.noSigningIssues())
-                b = SecurityWarning.showCertWarningDialog(
+                b = SecurityDialogs.showCertWarningDialog(
                         AccessType.SIGNING_ERROR, file, js);
             if (!b)
                 throw new LaunchException(null, null, R("LSFatal"),
@@ -527,7 +540,7 @@ public class JNLPClassLoader extends URLClassLoader {
      * loaded from the codebase are not cached.
      */
     public void enableCodeBase() {
-        addURL(file.getCodeBase()); // nothing happens if called more that once?
+        addToCodeBaseLoader(file.getCodeBase());
     }
 
     /**
@@ -584,7 +597,7 @@ public class JNLPClassLoader extends URLClassLoader {
                     (getCodeSourceSecurity(cs.getLocation()).getSecurityType().equals(SecurityDesc.ALL_PERMISSIONS) ||
                      getCodeSourceSecurity(cs.getLocation()).getSecurityType().equals(SecurityDesc.J2EE_PERMISSIONS))) {
 
-                permissions = getCodeSourceSecurity(cs.getLocation()).getPermissions();
+                permissions = getCodeSourceSecurity(cs.getLocation()).getPermissions(cs);
             }
 
             Enumeration<Permission> e = permissions.elements();
@@ -637,6 +650,7 @@ public class JNLPClassLoader extends URLClassLoader {
     protected void activateJars(final List<JARDesc> jars) {
         PrivilegedAction<Void> activate = new PrivilegedAction<Void>() {
 
+            @SuppressWarnings("deprecation")
             public Void run() {
                 // transfer the Jars
                 waitForJars(jars);
@@ -702,7 +716,7 @@ public class JNLPClassLoader extends URLClassLoader {
                                     List<JARDesc> jars = new ArrayList<JARDesc>();
                                     JARDesc jarDesc = new JARDesc(new File(extractedJarLocation).toURL(), null, null, false, false, false, false);
                                     jars.add(jarDesc);
-                                    tracker.addResource(new File(extractedJarLocation).toURL(), null, null);
+                                    tracker.addResource(new File(extractedJarLocation).toURL(), null, null, null);
                                     signer.verifyJars(jars, tracker);
 
                                     if (signer.anyJarsSigned() && !signer.getAlreadyTrustPublisher()) {
@@ -839,10 +853,10 @@ public class JNLPClassLoader extends URLClassLoader {
      * calls.
      */
     protected File getNativeDir() {
+        final int rand = (int)((Math.random()*2 - 1) * Integer.MAX_VALUE);
         nativeDir = new File(System.getProperty("java.io.tmpdir")
                              + File.separator + "netx-native-"
-                             + (new Random().nextInt() & 0xFFFF));
-
+                             + (rand & 0xFFFF));
         File parent = nativeDir.getParentFile();
         if (!parent.isDirectory() && !parent.mkdirs()) {
             return null;
@@ -956,8 +970,13 @@ public class JNLPClassLoader extends URLClassLoader {
             if (result != null)
                 return result;
         }
-
-        return null;
+        
+        // Result is still null. Return what the codebaseloader 
+        // has (which returns null if it is not loaded there either)
+        if (codeBaseLoader != null)
+            return codeBaseLoader.findLoadedClassFromParent(name);
+        else
+            return null;
     }
 
     /**
@@ -965,7 +984,7 @@ public class JNLPClassLoader extends URLClassLoader {
      * classloader, or one of the classloaders for the JNLP file's
      * extensions.
      */
-    public Class<?> loadClass(String name) throws ClassNotFoundException {
+    public synchronized Class<?> loadClass(String name) throws ClassNotFoundException {
 
         Class<?> result = findLoadedClassAll(name);
 
@@ -985,7 +1004,7 @@ public class JNLPClassLoader extends URLClassLoader {
         // validPackage(name);
 
         // search this and the extension loaders
-        if (result == null)
+        if (result == null) {
             try {
                 result = loadClassExt(name);
             } catch (ClassNotFoundException cnfe) {
@@ -1013,6 +1032,7 @@ public class JNLPClassLoader extends URLClassLoader {
 
                             tracker.addResource(desc.getLocation(),
                                     desc.getVersion(),
+                                    null,
                                     JNLPRuntime.getDefaultUpdatePolicy()
                                     );
 
@@ -1041,6 +1061,11 @@ public class JNLPClassLoader extends URLClassLoader {
                     }
                 }
             }
+        }
+
+        if (result == null) {
+            throw new ClassNotFoundException(name);
+        }
 
         return result;
     }
@@ -1060,6 +1085,11 @@ public class JNLPClassLoader extends URLClassLoader {
             }
         }
 
+        // Try codebase loader
+        if (codeBaseLoader != null)
+            return codeBaseLoader.findClass(name);
+        
+        // All else failed. Throw CNFE
         throw new ClassNotFoundException(name);
     }
 
@@ -1102,6 +1132,10 @@ public class JNLPClassLoader extends URLClassLoader {
         for (int i = 1; i < loaders.length; i++)
             if (result == null)
                 result = loaders[i].getResource(name);
+        
+        // If result is still null, look in the codebase loader
+        if (result == null && codeBaseLoader != null)
+            result = codeBaseLoader.getResource(name);
 
         return result;
     }
@@ -1113,9 +1147,9 @@ public class JNLPClassLoader extends URLClassLoader {
     @Override
     public Enumeration<URL> findResources(String name) throws IOException {
         Vector<URL> resources = new Vector<URL>();
+        Enumeration<URL> e;
 
         for (int i = 0; i < loaders.length; i++) {
-            Enumeration<URL> e;
 
             if (loaders[i] == this)
                 e = super.findResources(name);
@@ -1126,6 +1160,14 @@ public class JNLPClassLoader extends URLClassLoader {
                 resources.add(e.nextElement());
         }
 
+        // Add resources from codebase (only if nothing was found above, 
+        // otherwise the server will get hammered) 
+        if (resources.isEmpty() && codeBaseLoader != null) {
+            e = codeBaseLoader.findResources(name);
+            while (e.hasMoreElements())
+                resources.add(e.nextElement());
+        }
+
         return resources.elements();
     }
 
@@ -1243,6 +1285,9 @@ public class JNLPClassLoader extends URLClassLoader {
         // jars
         for (URL u : extLoader.getURLs())
             addURL(u);
+        
+        // Codebase
+        addToCodeBaseLoader(extLoader.file.getCodeBase());
 
         // native search paths
         for (File nativeDirectory : extLoader.getNativeDirectories())
@@ -1253,4 +1298,92 @@ public class JNLPClassLoader extends URLClassLoader {
             jarLocationSecurityMap.put(key, extLoader.jarLocationSecurityMap.get(key));
         }
     }
+
+    /**
+     * Adds the given path to the path loader
+     * 
+     * @param URL the path to add
+     * @throws IllegalArgumentException If the given url is not a path
+     */
+    private void addToCodeBaseLoader(URL u) {
+
+        // Only paths may be added
+        if (!u.getFile().endsWith("/")) {
+            throw new IllegalArgumentException("addToPathLoader only accepts path based URLs");
+        }
+
+        // If there is no loader yet, create one, else add it to the 
+        // existing one (happens when called from merge())
+        if (codeBaseLoader == null) {
+            codeBaseLoader = new CodeBaseClassLoader(new URL[] { u }, this);
+        } else {
+            codeBaseLoader.addURL(u);
+        }
+    }
+
+    private DownloadOptions getDownloadOptionsForJar(JARDesc jar) {
+        return file.getDownloadOptionsForJar(jar);
+    }
+
+    /*
+     * Helper class to expose protected URLClassLoader methods.
+     */
+
+    public static class CodeBaseClassLoader extends URLClassLoader {
+
+        JNLPClassLoader parentJNLPClassLoader;
+        
+        public CodeBaseClassLoader(URL[] urls, JNLPClassLoader cl) {
+            super(urls);
+            parentJNLPClassLoader = cl;
+        }
+
+        @Override
+        public void addURL(URL url) { 
+            super.addURL(url); 
+        }
+
+        @Override
+        public Class<?> findClass(String name) throws ClassNotFoundException { 
+            return super.findClass(name); 
+        }
+
+        /**
+         * Returns the output of super.findLoadedClass().
+         * 
+         * The method is renamed because ClassLoader.findLoadedClass() is final
+         * 
+         * @param name The name of the class to find
+         * @return Output of ClassLoader.findLoadedClass() which is the class if found, null otherwise 
+         * @see java.lang.ClassLoader#findLoadedClass(String)
+         */
+        public Class<?> findLoadedClassFromParent(String name) {
+            return findLoadedClass(name);
+        }
+
+        /**
+         * Returns JNLPClassLoader that encompasses this loader
+         * 
+         * @return parent JNLPClassLoader
+         */
+        public JNLPClassLoader getParentJNLPClassLoader() {
+            return parentJNLPClassLoader;
+        }
+
+        @Override
+        public Enumeration<URL> findResources(String name) throws IOException {
+            if (!name.startsWith("META-INF")) {
+                return super.findResources(name);
+            }
+            return (new Vector<URL>(0)).elements();
+        }
+
+        @Override
+        public URL findResource(String name) {
+            if (!name.startsWith("META-INF")) {
+                return super.findResource(name);
+            }
+            return null;
+        }
+    }
 }
