@@ -19,6 +19,7 @@ package net.sourceforge.jnlp.runtime;
 import java.io.*;
 import java.net.Authenticator;
 import java.net.ProxySelector;
+import java.nio.channels.FileChannel;
 import java.nio.channels.FileLock;
 import java.awt.*;
 import java.text.*;
@@ -35,8 +36,11 @@ import javax.swing.UIManager;
 import javax.swing.text.html.parser.ParserDelegator;
 
 import net.sourceforge.jnlp.*;
+import net.sourceforge.jnlp.browser.BrowserAwareProxySelector;
 import net.sourceforge.jnlp.cache.*;
+import net.sourceforge.jnlp.config.DeploymentConfiguration;
 import net.sourceforge.jnlp.security.JNLPAuthenticator;
+import net.sourceforge.jnlp.security.KeyStores;
 import net.sourceforge.jnlp.security.SecurityDialogMessageHandler;
 import net.sourceforge.jnlp.security.VariableX509TrustManager;
 import net.sourceforge.jnlp.services.*;
@@ -103,7 +107,7 @@ public class JNLPRuntime {
     private static boolean securityEnabled = true;
 
     /** whether debug mode is on */
-    private static boolean debug = false; // package access by Boot
+    private static boolean debug = false;
 
     /** whether streams should be redirected */
     private static boolean redirectStreams = false;
@@ -120,6 +124,9 @@ public class JNLPRuntime {
     /** contains the arguments passed to the jnlp runtime */
     private static List<String> initialArguments;
 
+    /** a lock which is held to indicate that an instance of netx is running */
+    private static FileLock fileLock;
+
     public static final String STDERR_FILE = "java.stderr";
     public static final String STDOUT_FILE = "java.stdout";
 
@@ -160,6 +167,8 @@ public class JNLPRuntime {
             }
         }
 
+        KeyStores.setConfiguration(config);
+
         initializeStreams();
 
         isWebstartApplication = isApplication;
@@ -178,8 +187,13 @@ public class JNLPRuntime {
         if (!headless && indicator == null)
             indicator = new DefaultDownloadIndicator();
 
-        if (handler == null)
-            handler = new DefaultLaunchHandler();
+        if (handler == null) {
+            if (headless) {
+                handler = new DefaultLaunchHandler();
+            } else {
+                handler = new GuiLaunchHandler();
+            }
+        }
 
         ServiceManager.setServiceManagerStub(new XServiceManagerStub()); // ignored if we're running under Web Start
 
@@ -217,7 +231,7 @@ public class JNLPRuntime {
 
         // plug in a custom authenticator and proxy selector
         Authenticator.setDefault(new JNLPAuthenticator());
-        ProxySelector.setDefault(new JNLPProxySelector());
+        ProxySelector.setDefault(new BrowserAwareProxySelector());
 
         // Restrict access to netx classes
         Security.setProperty("package.access", 
@@ -273,10 +287,10 @@ public class JNLPRuntime {
 
             try {
                 File errFile = new File(logDir, JNLPRuntime.STDERR_FILE);
-                errFile.getParentFile().mkdirs();
+                FileUtils.createParentDir(errFile);
                 FileUtils.createRestrictedFile(errFile, true);
                 File outFile = new File(logDir, JNLPRuntime.STDOUT_FILE);
-                outFile.getParentFile().mkdirs();
+                FileUtils.createParentDir(outFile);
                 FileUtils.createRestrictedFile(outFile, true);
 
                 if (redirectStreams) {
@@ -611,6 +625,23 @@ public class JNLPRuntime {
         }
     }
 
+    /**
+     * @return true if running on Windows
+     */
+    public static boolean isWindows() {
+        String os = System.getProperty("os.name");
+        return (os != null && os.startsWith("Windows"));
+    }
+
+    /**
+     * @return true if running on a Unix or Unix-like system (including Linux
+     * and *BSD)
+     */
+    public static boolean isUnix() {
+        String sep = System.getProperty("file.separator");
+        return (sep != null && sep.equals("/"));
+    }
+
     public static void setInitialArgments(List<String> args) {
         checkInitialized();
         SecurityManager securityManager = System.getSecurityManager();
@@ -623,4 +654,78 @@ public class JNLPRuntime {
         return initialArguments;
     }
 
+    /**
+     * Indicate that netx is running by creating the {@link JNLPRuntime#INSTANCE_FILE} and
+     * acquiring a shared lock on it
+     */
+    public synchronized static void markNetxRunning() {
+        if (fileLock != null) return;
+        try {
+            String message = "This file is used to check if netx is running";
+
+            File netxRunningFile = new File(JNLPRuntime.getConfiguration()
+                    .getProperty(DeploymentConfiguration.KEY_USER_NETX_RUNNING_FILE));
+            if (!netxRunningFile.exists()) {
+                FileUtils.createParentDir(netxRunningFile);
+                FileUtils.createRestrictedFile(netxRunningFile, true);
+                FileOutputStream fos = new FileOutputStream(netxRunningFile);
+                try {
+                    fos.write(message.getBytes());
+                } finally {
+                    fos.close();
+                }
+            }
+
+            FileInputStream is = new FileInputStream(netxRunningFile);
+            FileChannel channel = is.getChannel();
+            fileLock = channel.lock(0, 1, true);
+            if (!fileLock.isShared()){ // We know shared locks aren't offered on this system.
+                FileLock temp = null;
+                for (long pos = 1; temp == null && pos < Long.MAX_VALUE - 1; pos++){
+                    temp = channel.tryLock(pos, 1, false); // No point in requesting for shared lock.
+                }
+                fileLock.release(); // We can release now, since we hold another lock.
+                fileLock = temp; // Keep the new lock so we can release later.
+            }
+            
+            if (fileLock != null && fileLock.isShared()) {
+                if (JNLPRuntime.isDebug()) {
+                    System.out.println("Acquired shared lock on " +
+                            netxRunningFile.toString() + " to indicate javaws is running");
+                }
+            }
+        } catch (IOException e) {
+            e.printStackTrace();
+        }
+
+        Runtime.getRuntime().addShutdownHook(new Thread() {
+            public void run() {
+                markNetxStopped();
+                CacheUtil.cleanCache();
+            }
+        });
+    }
+
+    /**
+     * Indicate that netx is stopped by releasing the shared lock on
+     * {@link JNLPRuntime#INSTANCE_FILE}.
+     */
+    private static void markNetxStopped() {
+        if (fileLock == null) {
+            return;
+        }
+        try {
+            fileLock.release();
+            fileLock.channel().close();
+            fileLock = null;
+            if (JNLPRuntime.isDebug()) {
+                String file = JNLPRuntime.getConfiguration()
+                        .getProperty(DeploymentConfiguration.KEY_USER_NETX_RUNNING_FILE);
+                System.out.println("Release shared lock on " + file);
+            }
+        } catch (IOException e) {
+            e.printStackTrace();
+        }
+    }
+
 }
