@@ -20,22 +20,17 @@ import static net.sourceforge.jnlp.runtime.Translator.R;
 
 import java.awt.Frame;
 import java.awt.Window;
-import java.awt.event.WindowAdapter;
-import java.awt.event.WindowEvent;
 import java.lang.ref.WeakReference;
 import java.net.SocketPermission;
 import java.security.AllPermission;
 import java.security.AccessControlException;
-import java.security.AccessController;
 import java.security.Permission;
-import java.security.PrivilegedAction;
 import java.security.SecurityPermission;
-import java.util.PropertyPermission;
 
 import javax.swing.JWindow;
 
 import net.sourceforge.jnlp.JNLPFile;
-import net.sourceforge.jnlp.security.SecurityWarning.AccessType;
+import net.sourceforge.jnlp.security.SecurityDialogs.AccessType;
 import net.sourceforge.jnlp.services.ServiceUtil;
 import net.sourceforge.jnlp.util.WeakList;
 import sun.awt.AWTSecurityManager;
@@ -103,9 +98,6 @@ class JNLPSecurityManager extends AWTSecurityManager {
     private WeakList<ApplicationInstance> weakApplications =
             new WeakList<ApplicationInstance>();
 
-    /** weak reference to most app who's windows was most recently activated */
-    private WeakReference activeApplication = null;
-
     /** Sets whether or not exit is allowed (in the context of the plugin, this is always false) */
     private boolean exitAllowed = true;
 
@@ -204,8 +196,16 @@ class JNLPSecurityManager extends AWTSecurityManager {
 
         // this needs to be tightened up
         for (int i = 0; i < stack.length && i < maxDepth; i++) {
-            if (stack[i].getClassLoader() instanceof JNLPClassLoader) {
-                JNLPClassLoader loader = (JNLPClassLoader) stack[i].getClassLoader();
+            ClassLoader cl = stack[i].getClassLoader();
+            
+            // Since we want to deal with JNLPClassLoader, extract it if this 
+            // is a codebase loader
+            if (cl instanceof JNLPClassLoader.CodeBaseClassLoader)
+                cl = ((JNLPClassLoader.CodeBaseClassLoader) cl).getParentJNLPClassLoader();
+
+            if (cl instanceof JNLPClassLoader) {
+
+                JNLPClassLoader loader = (JNLPClassLoader) cl;
 
                 if (loader != null && loader.getApplication() != null) {
                     return loader.getApplication();
@@ -311,27 +311,6 @@ class JNLPSecurityManager extends AWTSecurityManager {
                             }
                         }
                     }
-
-                } else if (perm instanceof SecurityPermission) {
-                    tmpPerm = perm;
-
-                    // JCE's initialization requires putProviderProperty permission
-                    if (perm.equals(new SecurityPermission("putProviderProperty.SunJCE"))) {
-                        if (inTrustedCallChain("com.sun.crypto.provider.SunJCE", "run")) {
-                            return;
-                        }
-                    }
-
-                } else if (perm instanceof RuntimePermission) {
-                    tmpPerm = perm;
-
-                    // KeyGenerator's init method requires internal spec access
-                    if (perm.equals(new SecurityPermission("accessClassInPackage.sun.security.internal.spec"))) {
-                        if (inTrustedCallChain("javax.crypto.KeyGenerator", "init")) {
-                            return;
-                        }
-                    }
-
                 } else {
                     tmpPerm = perm;
                 }
@@ -355,34 +334,6 @@ class JNLPSecurityManager extends AWTSecurityManager {
         }
     }
 
-    /**
-     * Returns weather the given class and method are in the current stack,
-     * and whether or not everything upto then is trusted
-     *
-     * @param className The name of the class to look for in the stack
-     * @param methodName The name of the method for the given class to look for in the stack
-     * @return Weather or not class::method() are in the chain, and everything upto there is trusted
-     */
-    private boolean inTrustedCallChain(String className, String methodName) {
-
-        StackTraceElement[] stack = Thread.currentThread().getStackTrace();
-
-        for (int i = 0; i < stack.length; i++) {
-
-            // Everything up to the desired class/method must be trusted
-            if (!stack[i].getClass().getProtectionDomain().implies(new AllPermission())) {
-                return false;
-            }
-
-            if (stack[i].getClassName().equals(className) &&
-                    stack[i].getMethodName().equals(methodName)) {
-                return true;
-            }
-        }
-
-        return false;
-    }
-
     /**
      * Asks the user whether or not to grant permission.
      * @param perm the permission to be granted
@@ -495,13 +446,7 @@ class JNLPSecurityManager extends AWTSecurityManager {
         // but when they really call, stop only the app instead of the JVM
         ApplicationInstance app = getApplication(stack, 0);
         if (app == null) {
-            // should check caller to make sure it is JFrame.close or
-            // other known System.exit call
-            if (activeApplication != null)
-                app = (ApplicationInstance) activeApplication.get();
-
-            if (app == null)
-                throw new SecurityException(R("RExitNoApp"));
+            throw new SecurityException(R("RExitNoApp"));
         }
 
         app.destroy();
