@@ -75,7 +75,7 @@ import javax.swing.table.DefaultTableModel;
 import net.sourceforge.jnlp.security.CertificateUtils;
 import net.sourceforge.jnlp.security.KeyStores;
 import net.sourceforge.jnlp.security.SecurityUtil;
-import net.sourceforge.jnlp.security.SecurityWarningDialog;
+import net.sourceforge.jnlp.security.SecurityDialog;
 import net.sourceforge.jnlp.security.KeyStores.Level;
 import net.sourceforge.jnlp.util.FileUtils;
 
@@ -299,7 +299,7 @@ public class CertificatePane extends JPanel {
     }
 
     /** Allows storing KeyStores.Types in a JComponent */
-    private class CertificateType {
+    private static class CertificateType {
         private final KeyStores.Type type;
 
         public CertificateType(KeyStores.Type type) {
@@ -369,7 +369,11 @@ public class CertificatePane extends JPanel {
                     }
 
                     OutputStream os = new FileOutputStream(keyStoreFile);
-                    ks.store(os, KeyStores.getPassword());
+                    try {
+                        ks.store(os, KeyStores.getPassword());
+                    } finally {
+                        os.close();
+                    }
                     repopulateTables();
                 } catch (Exception ex) {
                     // TODO: handle exception
@@ -478,7 +482,7 @@ public class CertificatePane extends JPanel {
             int selectedRow = table.getSelectedRow();
             if (selectedRow != -1 && selectedRow >= 0) {
                 X509Certificate c = certs.get(selectedRow);
-                SecurityWarningDialog.showSingleCertInfoDialog(c, parent);
+                SecurityDialog.showSingleCertInfoDialog(c, parent);
             }
         }
     }
