@@ -220,6 +220,9 @@ NPError get_proxy_info(const char* siteAddr, char** proxy, uint32_t* len);
 void consume_message(gchar* message);
 void start_jvm_if_needed();
 static void appletviewer_monitor(GPid pid, gint status, gpointer data);
+void plugin_send_initialization_message(char* instance, gulong handle,
+                                               int width, int height,
+                                               char* url);
 
 // Global instance counter.
 // Mutex to protect plugin_instance_counter.
@@ -235,6 +238,8 @@ static GPid appletviewer_pid = -1;
 static guint appletviewer_watch_id = -1;
 
 int plugin_debug = getenv ("ICEDTEAPLUGIN_DEBUG") != NULL;
+int plugin_debug_suspend = (getenv("ICEDTEAPLUGIN_DEBUG") != NULL) &&
+        (strcmp(getenv("ICEDTEAPLUGIN_DEBUG"), "suspend") == 0);
 
 pthread_cond_t cond_message_available = PTHREAD_COND_INITIALIZER;
 
@@ -768,18 +773,9 @@ ITNP_SetWindow (NPP instance, NPWindow* window)
       data->window_height = window->height;
 
       // Now we have everything. Send this data to the Java side
-
-      gchar* instance_msg =  g_strdup_printf ("instance %s handle %ld width %d height %d %s",
-											 data->instance_id,
-											 (gulong) data->window_handle,
-											 data->window_width,
-											 data->window_height,
-											 data->applet_tag);
-
-      plugin_send_message_to_appletviewer (instance_msg);
-
-      g_free(instance_msg);
-      instance_msg = NULL;
+      plugin_send_initialization_message(
+    		  data->instance_id, (gulong) data->window_handle,
+    		  data->window_width, data->window_height, data->applet_tag);
 
       g_mutex_unlock (data->appletviewer_mutex);
 
@@ -1545,26 +1541,40 @@ plugin_start_appletviewer (ITNPPluginData* data)
   gchar** command_line;
   gchar** environment;
 
+  int cmd_num = 0;
   if (plugin_debug)
+  {
+      command_line = (gchar**) malloc(sizeof(gchar*)*11);
+      command_line[cmd_num++] = g_strdup(appletviewer_executable);
+      command_line[cmd_num++] = g_strdup(PLUGIN_BOOTCLASSPATH);
+      // set the classpath to avoid using the default (cwd).
+      command_line[cmd_num++] = g_strdup("-classpath");
+      command_line[cmd_num++] = g_strdup_printf("%s/lib/rt.jar", ICEDTEA_WEB_JRE);
+      command_line[cmd_num++] = g_strdup("-Xdebug");
+      command_line[cmd_num++] = g_strdup("-Xnoagent");
+      if (plugin_debug_suspend)
+      {
+          command_line[cmd_num++] = g_strdup("-Xrunjdwp:transport=dt_socket,address=8787,server=y,suspend=y");
+      } else
+      {
+          command_line[cmd_num++] = g_strdup("-Xrunjdwp:transport=dt_socket,address=8787,server=y,suspend=n");
+      }
+      command_line[cmd_num++] = g_strdup("sun.applet.PluginMain");
+      command_line[cmd_num++] = g_strdup(out_pipe_name);
+      command_line[cmd_num++] = g_strdup(in_pipe_name);
+      command_line[cmd_num] = NULL;
+  } else
   {
       command_line = (gchar**) malloc(sizeof(gchar*)*8);
-      command_line[0] = g_strdup(appletviewer_executable);
-      command_line[1] = g_strdup("-Xdebug");
-      command_line[2] = g_strdup("-Xnoagent");
-      command_line[3] = g_strdup("-Xrunjdwp:transport=dt_socket,address=8787,server=y,suspend=n");
-      command_line[4] = g_strdup("sun.applet.PluginMain");
-      command_line[5] = g_strdup(out_pipe_name);
-      command_line[6] = g_strdup(in_pipe_name);
-      command_line[7] = NULL;
-   } else
-   {
-       command_line = (gchar**) malloc(sizeof(gchar*)*5);
-       command_line[0] = g_strdup(appletviewer_executable);
-       command_line[1] = g_strdup("sun.applet.PluginMain");
-       command_line[2] = g_strdup(out_pipe_name);
-       command_line[3] = g_strdup(in_pipe_name);
-       command_line[4] = NULL;
-   }
+      command_line[cmd_num++] = g_strdup(appletviewer_executable);
+      command_line[cmd_num++] = g_strdup(PLUGIN_BOOTCLASSPATH);
+      command_line[cmd_num++] = g_strdup("-classpath");
+      command_line[cmd_num++] = g_strdup_printf("%s/lib/rt.jar", ICEDTEA_WEB_JRE);
+      command_line[cmd_num++] = g_strdup("sun.applet.PluginMain");
+      command_line[cmd_num++] = g_strdup(out_pipe_name);
+      command_line[cmd_num++] = g_strdup(in_pipe_name);
+      command_line[cmd_num] = NULL;
+  }
 
   environment = plugin_filter_environment();
 
@@ -1586,21 +1596,9 @@ plugin_start_appletviewer (ITNPPluginData* data)
 
   g_strfreev (environment);
 
-  g_free (command_line[0]);
-  command_line[0] = NULL;
-  g_free (command_line[1]);
-  command_line[1] = NULL;
-
-  if (plugin_debug)
-  {
-      g_free (command_line[2]);
-      command_line[2] = NULL;
-      g_free (command_line[3]);
-      command_line[3] = NULL;
-      g_free (command_line[4]);
-      command_line[4] = NULL;
-      g_free (command_line[5]);
-      command_line[5] = NULL;
+  for (int i = 0; i < cmd_num; i++) {
+    g_free (command_line[i]);
+    command_line[i] = NULL;
   }
 
   g_free(command_line);
@@ -1828,6 +1826,25 @@ plugin_send_message_to_appletviewer (gchar const* message)
   PLUGIN_DEBUG ("plugin_send_message_to_appletviewer return\n");
 }
 
+/*
+ * Sends the initialization message (handle/size/url) to the plugin
+ */
+void
+plugin_send_initialization_message(char* instance, gulong handle,
+                                   int width, int height, char* url)
+{
+  PLUGIN_DEBUG ("plugin_send_initialization_message\n");
+
+  gchar *window_message = g_strdup_printf ("instance %s handle %ld width %d height %d %s",
+                                            instance, handle, width, height, url);
+  plugin_send_message_to_appletviewer (window_message);
+  g_free (window_message);
+  window_message = NULL;
+
+  PLUGIN_DEBUG ("plugin_send_initialization_message return\n");
+}
+
+
 // Stop the appletviewer process.  When this is called the
 // appletviewer can be in any of three states: running, crashed or
 // hung.  If the appletviewer is running then sending it "shutdown"
@@ -1992,14 +2009,12 @@ NP_Initialize (NPNetscapeFuncs* browserTable, NPPluginFuncs* pluginTable)
 {
   PLUGIN_DEBUG ("NP_Initialize\n");
 
-  if (initialized)
-    return NPERR_NO_ERROR;
-  else if ((browserTable == NULL) || (pluginTable == NULL))
-    {
-      PLUGIN_ERROR ("Browser or plugin function table is NULL.");
+  if ((browserTable == NULL) || (pluginTable == NULL))
+  {
+    PLUGIN_ERROR ("Browser or plugin function table is NULL.");
 
-      return NPERR_INVALID_FUNCTABLE_ERROR;
-    }
+    return NPERR_INVALID_FUNCTABLE_ERROR;
+  }
 
   // Ensure that the major version of the plugin API that the browser
   // expects is not more recent than the major version of the API that
@@ -2106,6 +2121,12 @@ NP_Initialize (NPNetscapeFuncs* browserTable, NPPluginFuncs* pluginTable)
   pluginTable->getvalue = NPP_GetValueProcPtr (ITNP_GetValue);
 #endif
 
+  // Re-setting the above tables multiple times is OK (as the 
+  // browser may change its function locations). However 
+  // anything beyond this point should only run once.
+  if (initialized)
+    return NPERR_NO_ERROR;
+
   // Make sure the plugin data directory exists, creating it if
   // necessary.
   data_directory = g_strconcat (P_tmpdir, NULL);
@@ -2173,38 +2194,10 @@ NP_Initialize (NPNetscapeFuncs* browserTable, NPPluginFuncs* pluginTable)
     }
 
   // Set appletviewer_executable.
-  Dl_info info;
-  int filename_size;
-  if (dladdr ((const void*) ITNP_New, &info) == 0)
-    {
-      PLUGIN_ERROR_TWO ("Failed to determine plugin shared object filename",
-                        dlerror ());
-      np_error = NPERR_GENERIC_ERROR;
-      goto cleanup_data_directory;
-    }
-  filename = (gchar*) malloc(sizeof(gchar)*1024);
-  filename_size = readlink(info.dli_fname, filename, 1023);
-  if (filename_size >= 0)
-  {
-      filename[filename_size] = '\0';
-  }
-
-  if (!filename)
-    {
-      PLUGIN_ERROR ("Failed to create plugin shared object filename.");
-      np_error = NPERR_OUT_OF_MEMORY_ERROR;
-      goto cleanup_data_directory;
-    }
-
-  if (filename_size <= 0)
-  {
-      free(filename);
-      filename = g_strdup(info.dli_fname);
-  }
-
-  appletviewer_executable = g_strdup_printf ("%s/../../bin/java",
-                                             dirname (filename));
-  PLUGIN_DEBUG(".so is located at: %s and the link points to: %s. Executing java from dir %s to run %s\n", info.dli_fname, filename, dirname (filename), appletviewer_executable);
+  filename = g_strdup(ICEDTEA_WEB_JRE);
+  appletviewer_executable = g_strdup_printf ("%s/bin/java",
+                                             filename);
+  PLUGIN_DEBUG("Executing java at %s\n", appletviewer_executable);
   if (!appletviewer_executable)
     {
       PLUGIN_ERROR ("Failed to create appletviewer executable name.");
@@ -2436,11 +2429,7 @@ get_scriptable_object(NPP instance)
         // a 0 handle
         if (!data->window_handle)
         {
-            data->window_handle = 0;
-            gchar *window_message = g_strdup_printf ("instance %s handle %d",
-                                                    id_str, 0);
-            plugin_send_message_to_appletviewer (window_message);
-            g_free (window_message);
+            plugin_send_initialization_message(data->instance_id, 0, 0, 0, data->applet_tag);
         }
 
         java_result = java_request.getAppletObjectInstance(id_str);
